/* Binarna pretraga niza struktura - pretraga studenata po broju indeksa */

/* Datoteka sadrzi podatke o uspehu studenata na kolokvijumima iz osnova programiranja.
   Prva linija datoteke sadri broj studenata, a zatim svaka sledeca linija sadri ime i prezime
   odredjenog studenta, njegov broj indeksa (u obliku korisnickog imena na alas-u npr. mr01123) 
   i broj poena na prvom i na drugom kolokvijumu. Redosled studenata u datoteci je odredjen na 
   osnovu njihovog broja indeksa, i to tako da su na pocetku datoteke navedeni stariji studenti 
   sa manjim brojevima indeksa (npr. mv02234 je ispred mn03123 jer je stariji, a mr03123 je ispred 
   ml03234 jer ima manji broj indeksa).

   a) Definisati strukturu podataka za cuvanje podataka o studentima
   b) Napraviti funkciju koja poredi dva indeksa u skladu sa poretkom opisanim u uvodu zadatka
   c) Napsati rekurzivnu funckiju 

			int binary_search_recursive(char* indeks, .....) 				

      koja izracunava broj poena studenta sa datim brojem indeksa, ukoliko je student polagao 
      kolokvijum, odnosno -1 ukoliko nije.										
   d) Napisati iterativnu varijantu prethodne funkcije 

			int binary_search_iterative(char* indeks, ......)

   e) Napisati funkciju 

			int binary_search_stdlib(char* indeks, .....) 

      koja prethodnu funkcionalnost ostvaruje kroz poziv bibliotecke funkcije bsearch.
   f) Napisati program koji ucitava podatke o studentima iz datoteke cije je ime navedeno kao 
      argument komandne linije, zatim koristeci funkcije iz prethodnog dela zadatka ispisuje ukupan 
      broj poena za svakog studenata ciji indeks korisnik unese sa standardnog ulaza, sve dok ne unese 
      rec kraj za kraj.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Definicija strukture za cuvanje podataka o jednom studentu */
typedef struct _student
{
	char ime[15];
	char prezime[15];
	char indeks[8];
	int  kol_1;
	int  kol_2;
} student;

/* Studenti se smestaju u niz koji ce se dinamicki alocirati. 
   Pretpostavlja se da je niz sve vreme sortiran kako bi se 
   moglo vrsiti binarno pretrazivanje */
student* studenti;
int br_stud;

/* Funkcija vrsi poredjenje indeksa. Manjim se smatraju stariji studenti 
   sa manjim brojem indeksa. Funkca vraca negativnu vrednost ukoliko je 
   prvi indeks manji, pozitivnu ukoliko je veci, a 0 ukoliko su indeksi
   jednaki. Informacije o smerovima se u potpunosti zanemaruju.
*/
int poredi_indekse(char* indeks_1, char* indeks_2)
{
	/* Izdvaja se informacija o godini upisa i 
	   broju indeksa za oba studenta*/
	char s_godina_1[3], s_godina_2[3];
	int  godina_1, godina_2;
	char s_broj_1[4], s_broj_2[4];
	int  broj_1, broj_2;
	
	/* Godina upisa je zapisana u 3 i 4 karakteru
	   indeksa. Za nultu godinu uzimamo 1900. i smatramo da su 
	   studenti ciji je broj indeksa ispod 50 upisani sto godina posle. */

	strncpy(s_godina_1, indeks_1+2, 2);
	godina_1 = atoi(s_godina_1);
	if (godina_1 < 50) godina_1 += 100;

	strncpy(s_godina_2, indeks_2+2, 2);
	godina_2 = atoi(s_godina_2);
	if (godina_2 < 50) godina_2 += 100;
	
	/* Prvo poredimo godine */
	if (godina_1 < godina_2) 
		return -1;

	if (godina_1 > godina_2) 
		return 1;

	/* Ukoliko su godine jednake, izdvajamo brojeve indeksa 
	   koji su zapisani u 5. 6. i 7. karakteru indeksa */
	strncpy(s_broj_1, indeks_1+4, 3);
	broj_1 = atoi(s_broj_1);
	strncpy(s_broj_2, indeks_2+4, 3);		
	broj_2 = atoi(s_broj_2);			

	/* Umesto ovoga, prolazi i return strcmp(indeks_1+4, indeks_2+4); */

	/* Poredimo godine upisa */
	return broj_1-broj_2; 
}

/* Najjednostavnija rekurzivna implementacija algoritma 
   binarne pretrage */
int binary_search_recursive(char* indeks, int l, int d)
{
	int s, cmp;
	
	if (l>d)
		return -1;
		
	s = (l+d)/2;
	cmp = poredi_indekse(studenti[s].indeks, indeks);
	if (cmp == 0)
		return studenti[s].kol_1 + studenti[s].kol_2;
	else if (cmp < 0)
		return binary_search_recursive(indeks, s+1, d);
	else 
		return binary_search_recursive(indeks, l, s-1);
}

/* Iterativna varijanta prethodnog algoritma */
int binary_search_iterative(char* indeks)
{
	int l = 0, d = br_stud - 1;
	while (l <= d)
	{
		int s = (l+d)/2;
		int cmp = poredi_indekse(studenti[s].indeks, indeks);
		if (cmp == 0)
			return studenti[s].kol_1 + studenti[s].kol_2;
		else if (cmp < 0)
			l = s+1;
		else 
			d = s-1;
	}
	return -1;
}

/* Funkcija poredjenja koja je potrebna prilikom poziva 
   ugradjene funkcije bsearch */
int poredi(const void* indeks, const void* indeks_2)
{
	return poredi_indekse((char*)indeks, ((student*)indeks_2)->indeks);
}

int binary_search_stdlib(char* indeks)
{	
	student* s = bsearch(indeks, studenti, br_stud, sizeof(student), poredi);
	return s==NULL ? -1 : s->kol_1+s->kol_2;
}

/* Glavni program */
main(int argc, char* argv[])
{
	FILE* datoteka;
	int i;
	char indeks[8];
	
	/* Proveravamo prisutnost argumenata komandne linije */
	if (argc<2)
	{	printf("Upotreba %s : ime_datoteke\n",argv[0]);
		return -1;
	}


	/* Otvaramo datoteku */		
	if ((datoteka = fopen(argv[1], "r")) == NULL)
	{	printf("Greska prilikom otvaranja datoteke %s\n",argv[1]);
		return -1;
	}
	
	/* Ucivamo broj studenata i alociramo niz */
	fscanf(datoteka, "%d", &br_stud);
	studenti = (student*)malloc(br_stud*sizeof(student));
	
	/* Ucitavamo podatke o studentima */
	for (i = 0; i < br_stud; i++)
	{	fscanf(datoteka, "%s",studenti[i].ime);
		fscanf(datoteka, "%s",studenti[i].prezime);
		fscanf(datoteka, "%s",studenti[i].indeks);
		fscanf(datoteka, "%d", &studenti[i].kol_1);
		fscanf(datoteka, "%d", &studenti[i].kol_2);			
	}

	/* Ispisujemo poene za svakog studenta kojeg korisnik trazi */
	do
	{	printf("Unesi broj indeksa : ");
		scanf("%s",indeks);
		printf("Broj poena : %d\n",binary_search_stdlib(indeks));
	} while (strcmp(indeks,"kraj") != 0);


	/* Oslobadjamo alociranu memoriju */
	free(studenti);
	
	return 0;
	
}
