/* Sortiranje : program pronalazi sve anagrame medju unetim recima */
/* Dve reci su anagrami ako i samo ako se sastoje od istog broja istih slova */

/* Algoritam se zasniva na odredjivanju kanonske forme svake reci. 
   Kanonska forma se odredjuje sortiranjem slova svake reci. 
   Reci su anagrami ako i samo ako su im kanonske forme jednake.
   Sortiranjem niza reci na osnovu kanonskih formi anagrami se pojave 
   jedan uz drugi u nizu */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>


/* Pomocna funkcija koja ucitava rec. Pod recju se smatra niz slova */
int getword(char s[], int lim)
{
	int c;
	int i = 0;
	while (!isalpha(c = getchar()))
		if (c == EOF) 
			return 0;
	do
	{
		s[i++] = c;
	}
	while (i<lim-1 && isalpha(c = getchar()));

	s[i] = '\0';
	return i;
}



/* Maksimalna duzina reci */
#define MAX_WORD 50

/* Struktura reci je odredjena originalnom recju i njenim kanonskim predstavnikom */
struct word
{
	char original[MAX_WORD];
	char cannonical[MAX_WORD];
};


/* Funkcija koja vrsi poredjenje dva slova u reci */
int letter_compare(const void* a, const void* b)
{    return *(char*)a - *(char*)b;   }

/* Funkcija koja vrsi poredjenje po dve reci, na osnovu njihovih kanonskih formi*/
int cannonical_compare(const void* a, const void* b)
{
	return strcmp(((struct word*)a)->cannonical, 
		      ((struct word*)b)->cannonical);
}

/* Dve reci su anagrami ako imaju istu kanonsku formu a razlicite su */
int is_anagram(struct word a, struct word b)
{
	return strcmp(a.cannonical, b.cannonical) == 0 
	&& strcmp(a.original, b.original) != 0;
}

main()
{
	int i, n;
	struct word words[1000];

	/* Ucitavamo reci sa standardnog ulaza dok ih ima */
	for (i = 0; getword(words[i].original, MAX_WORD) != 0; i++)
	{
		/* Odredjujemo kanonsku formu svake reci */
		strcpy(words[i].cannonical, words[i].original);
		qsort(words[i].cannonical, strlen(words[i].cannonical), 
			sizeof(char), &letter_compare);
	}

	/* Broj ucitanih reci */	
	n = i;
	printf("Procitano %d reci\n", n);

	/* Sortiramo niz reci na osnovu kanonske forme */
	qsort(words, n, sizeof(struct word), &cannonical_compare);

	/* Pronalazimo anagrame kao susedne elemente niza i ispisujemo ih */		
	for (i = 1; i<n; i++)
		if (is_anagram(words[i-1], words[i]))
		{	printf("Anagram : %s - %s", 
				words[i-1].original, words[i].original);
			while(i+1 < n && is_anagram(words[i], words[i+1]))
			{	printf(" - %s", words[i+1].original);		
				i++;
			}	
			printf("\n");
		}

}
