/* Ubacivanje na pocetak jednostruko povezane liste - verzija sa **.
   Ispis i oslobadjanje liste realizovani iterativno. */

#include <stdio.h>
#include <stdlib.h>

typedef struct cvor
{
	int br;
	struct cvor* sl;
} CVOR;

/* Pomocna funkcija koja kreira cvor liste sa datim sadrzajem. 
   Funkcija kreira cvor i postavlja mu sadrzaj na dati broj. 
   Polje sl ostaje nedefinisano. 
   Funkcija vraca pokazivac na kreirani cvor. */

CVOR* napravi_cvor(int br)
{
	CVOR* novi = (CVOR*)malloc(sizeof(CVOR));
	if (novi == NULL)
	{
		fprintf(stderr, "Greska prilikom alokacije memorije\n");
		exit(1);
	}
	novi->br = br;
	return novi;
}

/* Zbog prenosa po vrednosti, sledeca funkcija ne radi ispravno */
/*
void ubaci_na_pocetak(CVOR* l, int br)
{
	CVOR* novi = napravi_cvor(br);
	novi->sl = l;
	l = novi;  /* Ovde se menja lokalna kopija pokazivaca l, a 
		      ne l iz funkcije pozivaoca (main) */
}
*/


/* Ubacuje dati broj na pocetak liste.  
   Pokazivac na pocetak liste se prenosi preko pokazivaca, umesto po 
   vrednosti, kako bi mogla da mu se izmeni vrednost. */
void ubaci_na_pocetak(CVOR** pl, int br)
{
	CVOR* novi = napravi_cvor(br);
	novi->sl = *pl;
	*pl = novi;
}

/* Ispisivanje liste : iterativna verzija */
void ispisi_listu(CVOR* l)
{
	CVOR* t;
	for (t = l; t != NULL; t=t->sl)
		printf("%d ", t->br);
}


/* Sledeca funkcija je neispravna */
/*
void oslobodi_listu(CVOR* l)
{
	CVOR* t;	
	for (t = l; t!=NULL; t = t->sl)
		free(t); 
		/* Ovde se unistava sadrzaj cvora na koji ukazuje t.
		   Korak petlje t = t->sl nece moci da se izvrsi */
	
}
*/

/* Oslobadjanje liste : iterativna verzija */
void oslobodi_listu(CVOR* l)
{
	while (l)
	{
		CVOR* tmp = l->sl;
		free(l);
		l = tmp;
	}
}

main()
{
	CVOR* l = NULL;
	int i;
	for (i = 0; i<10; i++)
		ubaci_na_pocetak(&l, i);

	ispisi_listu(l);
	putchar('\n');

	oslobodi_listu(l);
}
