/* Ubacivanje na pocetak jednostruko povezane liste - verzija sa 
   eksplicitnim vracanjem novog pocetka liste. 
   Ispis i oslobadjanje liste su realizovani rekurzivno 
*/ 

#include <stdio.h>
#include <stdlib.h>

typedef struct cvor
{
	int br;
	struct cvor* sl;
} CVOR;

/* Pomocna funkcija koja kreira cvor liste sa datim sadrzajem. 
   Funkcija kreira cvor i postavlja mu sadrzaj na dati broj. 
   Polje sl ostaje nedefinisano. 
   Funkcija vraca pokazivac na kreirani cvor. */

CVOR* napravi_cvor(int br)
{
	CVOR* novi = (CVOR*)malloc(sizeof(CVOR));
	if (novi == NULL)
	{
		fprintf(stderr, "Greska prilikom alokacije memorije\n");
		exit(1);
	}
	novi->br = br;
	return novi;
}

/* Ubacuje dati broj na pocetak date liste. 
   Funkcija pozivaocu eksplicitno vraca pocetak rezultujuce liste.*/
CVOR* ubaci_na_pocetak(CVOR* l, int br)
{
	CVOR* novi = napravi_cvor(br);
	novi->sl = l;
	return novi;
}

/* Ispisivanje liste : rekurzivna verzija */
void ispisi_listu(CVOR* l)
{
	if (l != NULL)
	{
		printf("%d ", l->br);
		ispisi_listu(l->sl);
	}
}


/* Ispisivanje liste unatrag : rekurzivna verzija */
/* Prethodna funkcija se lako modifikuje tako da ispisuje listu unazad */
void ispisi_listu_unazad(CVOR* l)
{
	if (l != NULL)
	{
		ispisi_listu_unazad(l->sl);
		printf("%d ", l->br);
	}
}

/* Oslobadjanje liste : rekurzivna verzija */
void oslobodi_listu(CVOR* l)
{
	if (l != NULL)
	{
		oslobodi_listu(l->sl);
		/* Prvo se oslobadja poslednji element liste */
		/* printf("Oslobadjam %d\n", l->br); */
		free(l);
	}
}

main()
{
	CVOR* l = NULL;
	int i;
	for (i = 0; i<10; i++)
		l = ubaci_na_pocetak(l, i);

	ispisi_listu(l);
	putchar('\n');

	ispisi_listu_unazad(l);
	putchar('\n');

	oslobodi_listu(l);
}
