/* Ubacivanje na kraj jednostruko povezane liste 
   - verzija sa ** 
   - iterativna i rekurzivna verzija 
*/

#include <stdio.h>
#include <stdlib.h>

typedef struct cvor
{
	int br;
	struct cvor* sl;
} CVOR;

/* Pomocna funkcija koja kreira cvor liste sa datim sadrzajem. 
   Funkcija kreira cvor i postavlja mu sadrzaj na dati broj. 
   Polje sl ostaje nedefinisano. 
   Funkcija vraca pokazivac na kreirani cvor. */

CVOR* napravi_cvor(int br)
{
	CVOR* novi = (CVOR*)malloc(sizeof(CVOR));
	if (novi == NULL)
	{
		fprintf(stderr, "Greska prilikom alokacije memorije\n");
		exit(1);
	}
	novi->br = br;
	return novi;
}

/* Ubacuje dati broj na pocetak liste.  
   Pokazivac na pocetak liste se prenosi preko pokazivaca, umesto po 
   vrednosti, kako bi mogla da mu se izmeni vrednost. 
   Iterativna verzija funkcije */
/* Ubacivanje na kraj liste je neefikasna operacija */
void ubaci_na_kraj(CVOR** pl, int br)
{
	CVOR* novi = napravi_cvor(br);
	novi->sl = 0;
	
	if (*pl == NULL)
		*pl = novi;
	else
	{
		/* Pronalazimo poslednji element liste - t*/
		CVOR* t;
		for (t=*pl; t->sl!=NULL; t=t->sl)
			;
		t->sl = novi;
	}
		
}

/* Rekurzivna varijanta prethodne funkcije */
void ubaci_na_kraj_rekurzivno(CVOR** pl, int br)
{
	if (*pl == NULL)
	{
		CVOR* novi = napravi_cvor(br);
		*pl = novi;
	}
	else
		ubaci_na_kraj_rekurzivno( &((*pl)->sl) ,br);
}

/* Ispisivanje liste : iterativna verzija */
void ispisi_listu(CVOR* l)
{
	CVOR* t;
	for (t = l; t != NULL; t=t->sl)
		printf("%d ", t->br);
}


/* Iterativna verzija funkcije koja oslobadja listu */
void oslobodi_listu(CVOR* l)
{
	while (l)
	{
		CVOR* tmp = l->sl;
		free(l);
		l = tmp;
	}
}

main()
{
	CVOR* l = NULL;
	int i;
	for (i = 0; i<5; i++)
		ubaci_na_kraj(&l, i);
	for (; i<10; i++)
		ubaci_na_kraj_rekurzivno(&l, i);

	ispisi_listu(l);
	putchar('\n');

	oslobodi_listu(l);
}
