/* Ubacivanje na kraj jednostruko povezane liste 
  - verzija sa eksplicitnim vracanjem nove liste 
  - iterativna i rekurzivna verzija */

#include <stdio.h>
#include <stdlib.h>

typedef struct cvor
{
	int br;
	struct cvor* sl;
} CVOR;

/* Pomocna funkcija koja kreira cvor liste sa datim sadrzajem. 
   Funkcija kreira cvor i postavlja mu sadrzaj na dati broj. 
   Polje sl ostaje nedefinisano. 
   Funkcija vraca pokazivac na kreirani cvor. */

CVOR* napravi_cvor(int br)
{
	CVOR* novi = (CVOR*)malloc(sizeof(CVOR));
	if (novi == NULL)
	{
		fprintf(stderr, "Greska prilikom alokacije memorije\n");
		exit(1);
	}
	novi->br = br;
	return novi;
}

/* Funkcija vraca pocetak rezultujuce liste */
CVOR* ubaci_na_kraj(CVOR* l, int br)
{
	CVOR* novi = napravi_cvor(br);
	novi->sl = NULL;
	
	if (l == NULL)
		return novi;
	else
	{
		CVOR* t;	
		for (t = l; t->sl!=NULL; t=t->sl)
			;
		t->sl = novi;

		/* Pocetak se nije promenio */
		return l;
	}
}

/* Rekurzivna varijanta prethodne funkcije.
   I ova funkcija vraca pokazivac na pocetak rezultujuce liste  */
CVOR* ubaci_na_kraj_rekurzivno(CVOR* l, int br)
{
	if (l == NULL)
	{
		CVOR* novi = napravi_cvor(br);
		return novi;
	}
	
	l->sl = ubaci_na_kraj_rekurzivno(l->sl, br);
	return l;
}

/* Ispisivanje liste : iterativna verzija */ 
void ispisi_listu(CVOR* l) 
{
	CVOR* t;
	for (t = l; t != NULL; t=t->sl)
		printf("%d ", t->br);
}


/* Iterativna verzija funkcije koja oslobadja listu */
void oslobodi_listu(CVOR* l)
{
	while (l)
	{
		CVOR* tmp = l->sl;
		free(l);
		l = tmp;
	}
}

main()
{
	CVOR* l = NULL;
	int i;
	for (i = 0; i<5; i++)
		l = ubaci_na_kraj(l, i);
	for (; i<10; i++)
		l = ubaci_na_kraj_rekurzivno(l, i);

	ispisi_listu(l);
	putchar('\n');

	oslobodi_listu(l);
}
