/* Ubacivanje na odgovarajuce mesto sortirane jednostruko povezane liste 
   - verzija sa ** 
   - iterativna i rekurzivna verzija 
*/

#include <stdio.h>
#include <stdlib.h>

typedef struct cvor
{
	int br;
	struct cvor* sl;
} CVOR;

/* Pomocna funkcija koja kreira cvor liste sa datim sadrzajem. 
   Funkcija kreira cvor i postavlja mu sadrzaj na dati broj. 
   Polje sl ostaje nedefinisano. 
   Funkcija vraca pokazivac na kreirani cvor. */

CVOR* napravi_cvor(int br)
{
	CVOR* novi = (CVOR*)malloc(sizeof(CVOR));
	if (novi == NULL)
	{
		fprintf(stderr, "Greska prilikom alokacije memorije\n");
		exit(1);
	}
	novi->br = br;
	return novi;
}
/* Kljucna ideja u realizaciji ove funkcije je pronalazenje poslednjeg 
   elementa liste ciji je kljuc manji od datog elementa br. 
*/
void ubaci_sortirano(CVOR** pl, int br)
{
	CVOR* novi = napravi_cvor(br);

	/* U sledeca dva slucaja ne postoji cvor ciji je kljuc manji 
	   od datog broja (br)
	   - Prvi je slucaj prazne liste 
	   - Drugi je slucaj kada je br manji od prvog elementa 
		
	   U oba slucaja ubacujemo na pocetak liste.
	*/
	if (*pl == NULL || br < (*pl)->br)
	{	
		novi->sl = *pl;
		*pl = novi;
		return;
	}

	/* Krecemo od pocetka i idemo dalje sve dok t nije poslednji 
	   manji element liste ili eventualno bas poslednji */
	CVOR* t;
	for (t = *pl; t->sl!=NULL && t->sl->br < br; t=t->sl)
		;
	novi->sl = t->sl;
	t->sl = novi;
}

/* Rekurzivna verzija prethodne funkcije */
void ubaci_sortirano_rekurzivno(CVOR** pl, int br)
{
	if (*pl == NULL || br < (*pl)->br)
	{
		CVOR* novi = napravi_cvor(br);
		novi->sl = *pl;
		*pl = novi;
		return;
	}

	ubaci_sortirano(&((*pl)->sl), br);
}

/* Ispisivanje liste : iterativna verzija */
void ispisi_listu(CVOR* l)
{
	CVOR* t;
	for (t = l; t != NULL; t=t->sl)
		printf("%d ", t->br);
}


/* Iterativna verzija funkcije koja oslobadja listu */
void oslobodi_listu(CVOR* l)
{
	while (l)
	{
		CVOR* tmp = l->sl;
		free(l);
		l = tmp;
	}
}

main()
{
	CVOR* l = NULL;
	CVOR* k = NULL;
	int i;

	ubaci_sortirano(&l, 5);
	ubaci_sortirano(&l, 8);
	ubaci_sortirano(&l, 7);
	ubaci_sortirano(&l, 6);
	ubaci_sortirano(&l, 4);

	ubaci_sortirano_rekurzivno(&k, 5);
	ubaci_sortirano_rekurzivno(&k, 8);
	ubaci_sortirano_rekurzivno(&k, 7);
	ubaci_sortirano_rekurzivno(&k, 6);
	ubaci_sortirano_rekurzivno(&k, 4);

	ispisi_listu(l);
	putchar('\n');

	ispisi_listu(k);
	putchar('\n');

	oslobodi_listu(l);
}
