/* Provera uparenosti HTML etiketa - stek se implementira preko liste */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

/* Maksimalna duzina etikete */
#define MAX_TAG 100

#define OPEN   1
#define CLOSED 2
#define ERROR 0

/* Funkcija ucitava sledecu etiketu i smesta njen naziv u niz s duzine max.
   Vraca OPEN za otvorenu etiketu, CLOSED za zatvorenu etiketu, 
   odnosno ERROR inace */
int gettag(char s[], int max)
{   int c, i;
    int type=OPEN;
    
    /* Preskacemo sve do znaka '<' */
    while ((c=getchar())!=EOF && c!='<')
                   ;
    /* Nismo naisli na etiketu */
    if (c==EOF)
           return ERROR;

    /* Proveravamo da li je etiketa zatvorena */
    if ((c=getchar())=='/')
       type = CLOSED;
    else
       ungetc(c,stdin);
    
    /* Citamo etiketu dok nailaze slova i smestamo ih u nisku*/ 
    for (i=0; isalpha(c=getchar()) && i<max-1; s[i++] = c)
	                 ;
    s[i]='\0';

    /* Preskacemo atribute do znaka > */
    while (c!=EOF && c!='>')
	c = getchar();

    /* Greska ukoliko nismo naisli na '>' */
    return c=='>' ? type : ERROR;
}

/***********************************************************************/
/* Stek ce biti implementiran koriscenjem liste */
typedef struct node
{
	char string[MAX_TAG];
	struct node* next;
} NODE;

/* Funkcija postavlja dati string na stek */
void push(NODE** pstack, char* s)
{
	NODE* tmp = (NODE*)malloc(sizeof(NODE));
	if (tmp == NULL)
	{
		fprintf(stderr, "Greska prilikom alokacije memorije\n");
		exit(1);
	}
	strcpy(tmp->string, s);
	tmp->next = *pstack;
	*pstack = tmp;
}

/* Funkcija cita podatak sa vrha steka */
char* peek(NODE* stack)
{
	/* Stek ne sme da bude prazan */
	assert(stack != NULL);

	return stack->string;
}

/* Funkcija uklanja podatak sa vrha steka */
void pop(NODE** pstack)
{
	/* Ukoliko je stek prazan ne radimo nista */
	if (*pstack == NULL)
		return;

	NODE* tmp = (*pstack)->next;
	free(*pstack);
	*pstack = tmp;
}

/* Funkcija proverava da li je dati stek prazan */
int empty(NODE* stack)
{
	return stack == NULL;
}
/* ********************************************************************* */


main()
{   
	char tag[MAX_TAG];
	NODE* stack = NULL;

	int type;
	while ((type = gettag(tag,MAX_TAG)) != ERROR)
	{
		if (type == OPEN)
		{
			/* Svaku otvorenu etiketu stavljamo na stek */
			push(&stack, tag);
			printf("Postavio <%s> na stek\n", peek(stack));
		}
		else
		{
			/* Za zatvorene etikete proveravamo da li je stek prazan 
			odnosno da li se na vrhu steka nalazi odgovarajuca otvorena etiketa */
			if (!empty(stack) && strcmp(peek(stack), tag) == 0)
			{
				printf("Skidam <%s> sa steka\n", peek(stack));
				/* Uklanjamo etiketu sa steka */
				pop(&stack);
			}
			else
			{
				/* Prijavljujemo gresku */
				printf("Neodgovarajuce : </%s>\n",tag);
				exit(1);
			}
		}
	}
	
	/* Proveravamo da li je stack ispraznjen */
	if (!empty(stack))
		fprintf(stderr, "Nisu sve etikete zatvorene\n");    
}
