/* Binarno pretrazivacko drvo - drvo sadrzi cele brojeve. 
   Ubacivanje realizovano preko ** */

#include <stdlib.h>
#include <stdio.h>

/* Struktura jednog cvora drveta */
typedef struct _cvor
{
	/* Podatak */
	int broj;
	/* Pokazivac na levo i desno podstablo */
	struct _cvor *l, *d;
} cvor;

/* Pomocna funkcija koja kreira novi cvor na osnovu datog broja */
cvor* napravi_cvor(int b)
{
	cvor* novi = (cvor*)malloc(sizeof(cvor));
	if (novi == NULL)
	{
		fprintf(stderr, "Greska prilikom alokacije memorije");
		exit(1);
	}
	
	/* Postavljamo brojnu vrednost */	
	novi->broj = b;

	/* Novi cvor se kreira kao list */
	novi->l = NULL;
	novi->d = NULL;
	return novi;
}

/* Rekurzivna funkcija koja ubacuje dati broj u dato drvo */
void ubaci_u_drvo(cvor** pdrvo, int b)
{
	/* Ukoliko je drvo prazno kreira se novi cvor */
	if (*pdrvo == NULL)
	{
		*pdrvo = napravi_cvor(b);
		return;
	}

	/* Ukoliko je broj koji se ubacuje manji od broja u korenu, 
	   rekurzivno ga ubacujemo u levo podstablo. 
	   Ukoliko je broj koji se ubacuje veci od broja u korenu, 
	   rekurzivno ga ubacujemo u desno podstablo. 
	*/

	if (b < (*pdrvo)->broj)
		ubaci_u_drvo(&((*pdrvo)->l), b);
	else if (b > (*pdrvo)->broj)
		ubaci_u_drvo(&((*pdrvo)->d), b);
}


/* Rekurzivna funkcija koja proverava da li dati broj postoji u drvetu */
int pronadji(cvor* drvo, int b)
{	
	/* U praznom drvetu ne postoji broj */
	if (drvo == NULL)
		return 0;
	
	/* Ukoliko je jednak vrednosti u korenu, onda postoji */
	if (drvo->broj == b)
		return 1;

	/* Ukoliko je broj koji trazimo manji od vrednosti u korenu, 
	   trazimo ga samo u levom podstablu, a inace ga trazimo 
	   samo u desnom podstablu */
	if (b < drvo->broj)
		return pronadji(drvo->l, b);
	else
		return pronadji(drvo->d, b);
}

/* Rekurzivna funkcija koja ispisuje drvo u inorder redosledu */
void ispisi_drvo(cvor* drvo)
{
	if (drvo != NULL)
	{
		ispisi_drvo(drvo->l);
		printf("%d ", drvo->broj);
		ispisi_drvo(drvo->d);
	}
}


/* Rekurzivna funkcija koja uklanja drvo. Obilazak mora biti postorder. */
void obrisi_drvo(cvor* drvo)
{
	if (drvo != NULL)
	{
		obrisi_drvo(drvo->l);
		obrisi_drvo(drvo->d);
		free(drvo);
	}
}

main()
{
	cvor* drvo = NULL;
	ubaci_u_drvo(&drvo, 1);
	ubaci_u_drvo(&drvo, 8);
	ubaci_u_drvo(&drvo, 3);
	ubaci_u_drvo(&drvo, 5);
	ubaci_u_drvo(&drvo, 7);
	ubaci_u_drvo(&drvo, 6);
	ubaci_u_drvo(&drvo, 9);

	if (pronadji(drvo, 3))
		printf("Pronadjeno 3\n");
	if (pronadji(drvo, 2))
		printf("Pronadjeno 2\n");
	if (pronadji(drvo, 7))
		printf("Pronadjeno 7\n");

	ispisi_drvo(drvo);
	putchar('\n');

	obrisi_drvo(drvo);	
}
