/* Rekurzivne funkcije za rad sa celobrojnim stablima (ne obavezno  pretrazivackim): 
   broj_cvorova, broj_listova, suma_cvorova, dubina, najveci_cvor, ... */

#include <stdlib.h>
#include <stdio.h>

typedef struct _cvor
{
	int broj;
	struct _cvor *l, *d;
} cvor;

cvor* napravi_cvor(int b)
{
	cvor* novi = (cvor*)malloc(sizeof(cvor));
	if (novi == NULL)
	{
		fprintf(stderr, "Greska prilikom alokacije memorije");
		exit(1);
	}
	novi->broj = b;
	novi->l = NULL;
	novi->d = NULL;
	return novi;
}

void ubaci_u_drvo(cvor** drvo, int b)
{
	if (*drvo == NULL)
	{
		*drvo = napravi_cvor(b);
		return;
	}

	if (b < (*drvo)->broj)
		ubaci_u_drvo(&((*drvo)->l), b);
	else
		ubaci_u_drvo(&((*drvo)->d), b);
}

void ispisi_drvo(cvor* drvo)
{
	if (drvo != NULL)
	{
		ispisi_drvo(drvo->l);
		printf("%d ", drvo->broj);
		ispisi_drvo(drvo->d);
	}
}


void obrisi_drvo(cvor* drvo)
{
	if (drvo != NULL)
	{
		obrisi_drvo(drvo->l);
		obrisi_drvo(drvo->d);
		free(drvo);
	}
}

/* Izracunava sumu svih elemenata u cvorovima drveta */
int suma_cvorova(cvor* drvo)
{
	if (drvo == NULL)
		return 0;
	return  suma_cvorova(drvo->l) +
		drvo->broj + 
		suma_cvorova(drvo->d);
}

/* Izracunava broj cvorova datog drveta */
int broj_cvorova(cvor* drvo)
{
	if (drvo == NULL)
		return 0;
	return  broj_cvorova(drvo->l) + 
		1 + 
		broj_cvorova(drvo->d);
}

/* Izracunava broj listova datog drveta */
int broj_listova(cvor* drvo)
{
	if (drvo == NULL)
		return 0;

	/* Cvor je list ukoliko nema ni jednog naslednika */
	if (drvo->l == NULL && drvo->d == NULL)
		return 1;

	return  broj_listova(drvo->l) + 
		broj_listova(drvo->d);
}

/* Izracunava sumu svih elemenata u listovima drveta */
int suma_listova(cvor* drvo)
{
	if (drvo == NULL)
		return 0;

	if (drvo->l == NULL && drvo->d == NULL)
		return drvo->broj;

	return  suma_listova(drvo->l) + 
		suma_listova(drvo->d);	
}

/* Ispisuje sve elemente u listovima drveta */
void ispisi_listove(cvor* drvo)
{
	if (drvo == NULL)
		return;

	ispisi_listove(drvo->l);

	if (drvo->l == NULL && drvo->d == NULL)
		printf("%d ", drvo->broj);

	ispisi_listove(drvo->d);
}

/* Izracunava vrednost najveceg cvora u proizvoljnom drvetu. 
   Vraca -1 ukoliko je drvo prazno */
int najveci_cvor(cvor* drvo)
{

	if (drvo == NULL)
		return -1;
	else
	{
		int max_l = najveci_cvor(drvo->l);
		int max_d = najveci_cvor(drvo->d);

		return max_l<max_d ?
			(max_d<drvo->broj?drvo->broj:max_d) :
			(max_l<drvo->broj?drvo->broj:max_l);
	}
}

/* Izracunava dubinu (broj nivoa drveta) */
int dubina(cvor* drvo)
{
	if (drvo == NULL)
		return 0;
	else 
	{
		int dl = dubina(drvo->l);
		int dd = dubina(drvo->d);

		return dl<dd ? dd + 1 : dl + 1;
	}
}

main()
{
	cvor* drvo = NULL;
	ubaci_u_drvo(&drvo, 1);
	ubaci_u_drvo(&drvo, 8);
	ubaci_u_drvo(&drvo, 5);
	ubaci_u_drvo(&drvo, 3);
	ubaci_u_drvo(&drvo, 7);
	ubaci_u_drvo(&drvo, 6);
	ubaci_u_drvo(&drvo, 9);

	printf("Suma cvorova : %d\n", suma_cvorova(drvo));
	printf("Broj cvorova : %d\n", broj_cvorova(drvo));
	printf("Broj listova : %d\n", broj_listova(drvo));
	printf("Suma listova : %d\n", suma_listova(drvo));
	printf("Najveci cvor : %d\n", najveci_cvor(drvo));
	printf("Dubina : %d\n", dubina(drvo));
	printf("Listovi : ");
	ispisi_listove(drvo);
	putchar('\n');

	obrisi_drvo(drvo);	
}
