/* Program sa ulaza cita tekst i ispisuje broj pojavljivanja svake od reci 
   koje su se javljale u tekstu.
   Verzija sa binarnim pretrazivackim drvetom.
*/

#include <stdlib.h>
#include <stdio.h>

/* Cvor drveta sadrzi ime reci i broj njenih pojavljivanja */
typedef struct _cvor
{    char ime[80];
     int br_pojavljivanja;
     struct _cvor* levo, *desno;
} cvor;


/* Funkcija ispisuje drvo u inorder redosledu */
void ispisi_drvo(cvor* drvo)
{    if (drvo!=NULL)
     {  ispisi_drvo(drvo->levo);
        printf("%s %d\n",drvo->ime,drvo->br_pojavljivanja);
        ispisi_drvo(drvo->desno);
     }
}


/* Funkcija uklanja binarno drvo iz memorije */
void obrisi_drvo(cvor* drvo)
{    if (drvo!=NULL)
     {  obrisi_drvo(drvo->levo);
        obrisi_drvo(drvo->desno);
        free(drvo);
     }
}

/* Funkcija ubacuje datu rec u dato drvo i vraca pokazivac na koren drveta */
cvor* ubaci(cvor* drvo, char rec[])
{ 
    /* Ukoliko je drvo prazno gradimo novi cvor */
    if (drvo==NULL)
    {   cvor* novi_cvor=(cvor*)malloc(sizeof(cvor));
        if (novi_cvor==NULL)
        {  printf("Greska prilikom alokacije memorije\n");
           exit(1);
        }
        strcpy(novi_cvor->ime, rec);
        novi_cvor->br_pojavljivanja=1;
        return novi_cvor;
    }
    int cmp = strcmp(rec, drvo->ime);
 
    /* Ukoliko rec vec postoji u drvetu uvecavamo njen broj pojavljivanja */
    if (cmp==0)
    {   drvo->br_pojavljivanja++;
        return drvo;
    }

    /* Ukoliko je rec koju ubacujemo leksikografski ispred reci koja je u 
       korenu drveta, rec ubacujemo u levo podstablo */
    if (cmp<0)
    {   drvo->levo=ubaci(drvo->levo, rec); 
        return drvo;
    }

    /* Ukoliko je rec koju ubacujemo leksikografski iza reci koja je u 
       korenu drveta, rec ubacujemo u desno podstablo */
    if (cmp>0)
    {   drvo->desno=ubaci(drvo->desno, rec); 
        return drvo;
    }
}

/* Pomocna funkcija koja cita rec sa standardnog ulaza i vraca njenu 
   duzinu, odnosno -1 ukoliko se naidje na EOF */
int getword(char word[], int lim) 
{   int c, i=0;
    while (!isalpha(c=getchar()) && c!=EOF)
             ;

    if (c==EOF)
          return -1;
    do
    {   word[i++]=c;
    }while (i<lim-1 && isalpha(c=getchar()));

    word[i]='\0';
    return i;
}

main()
{
   /* Drvo je na pocetku prazno */
   cvor* drvo=NULL;
   char procitana_rec[80];

   /* Citamo rec po rec dok ne naidjemo na kraj datoteke i 
      ubacujemo ih u drvo */
   while(getword(procitana_rec,80)!=-1)
      drvo=ubaci(drvo,procitana_rec);

   /* Ispisujemo drvo */
   ispisi_drvo(drvo);
 
   /* Uklanjamo ga iz memorije */
   obrisi_drvo(drvo);

}

