/* MINESWEEPER - primer jednostavne igrice */
/* Program demonstrira rad sa matricama, slucajnim brojevima i 
   rekurzivnu implementaciju DFS algoritma za obilazak grafova */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

/* Dimenzija table */
int   n;

/* Tabla koja sadrzi 0 i 1 u zavisnosti od toga da li na polju postoji bomba */
int** bombe;

/* Tabla koja opisuje tekuce stanje igre. Moze da sadrzi sledece vrednosti :
    ZATVORENO - opisuje polje koje jos nije bilo otvarano
    PRAZNO - polje na kome ne postoji ni jedna bomba
    broj od 1-8 - polje koje je otvoreno i na kome pise koliko bombi postoji u okolini
    ZASTAVICA - polje koje je korisnik oznacio zastavicom
*/
#define PRAZNO (-1)
#define ZATVORENO 0
#define ZASTAVICA 9

int** stanje;

/* Ukupan broj bombi */
int broj_bombi;

/* Ukupan broj postavljenih zastavica */
int broj_zastavica = 0;


/* Pomocne funkcije za rad sa matricama */
int** alociraj(int n)
{  int i;
   int** m=malloc(n*sizeof(int*));
   for (i=0; i<n; i++)
       m[i]=calloc(n,sizeof(int));
   return m; 
}

void obrisi(int** m, int n)
{   int i;
    for (i=0; i<n; i++)
       free(m[i]);

    free(m);
}

/* Funkcija postavlja bombe */ 
void postavi_bombe()
{  broj_bombi=(n*n)/6;
   int kolona;
   int vrsta;
   int i;

   /* Inicijalizujemo generator slucajnih brojeva */
   srand(time(NULL));

   for (i=0; i<broj_bombi; i++)
   {    /* Racunamo slucajni polozaj bombe */
        kolona=rand()%n;
        vrsta=rand()%n;

              
        /* Ukoliko bomba vec postoji tu, opet idemo u istu iteraciju */
        if (bombe[vrsta][kolona]==1)
        {   i--; 
            continue;
        }        

        /* Postavljamo bombu */
        bombe[vrsta][kolona]=1;   
  }
}


/* Funkcija ispisuje tablu sa bombama */
void ispisi_bombe()
{   int i,j;
    for (i=0; i<n; i++)
    {   for (j=0; j<n; j++)
            printf("%d",bombe[i][j]);
        printf("\n");
    }
}

/* Funkcija ispisuje tekuce stanje */
void ispisi_stanje()
{   int i,j;

    /* Brisemo ekran pozivajuci komandu operativnog sistema */
    system("clear");


    for (i=0; i<n; i++)
    {   for (j=0; j<n; j++)
        {    if (stanje[i][j]==ZATVORENO)
                  printf(".");
             else if (stanje[i][j]==PRAZNO)
                  printf(" ");
             else if (stanje[i][j]==ZASTAVICA)
                  printf("*");
             else  
                  printf("%d",stanje[i][j]);
        }
        printf("\n");
    }

}

/* Funkcija postavlja zastavicu na dato polje ili je uklanja 
   ukoliko vec postoji */
void postavi_zastavicu(int i, int j)
{    if (stanje[i][j]==ZATVORENO)
     {    stanje[i][j]=ZASTAVICA;
          broj_zastavica++;
     }
     else if (stanje[i][j]==ZASTAVICA)
     {    stanje[i][j]=ZATVORENO;
          broj_zastavica--;
     }
}

/* Funkcija izracunava koliko bombi postoji u okolini date bombe */
int broj_bombi_u_okolini(int v, int k)
{    int i, j;
     int br=0;
     /* Prolazimo kroz sva okolna polja */
     for (i=-1; i<=1; i++)
         for(j=-1; j<=1; j++)
         {  /* preskacemo centralno polje */
            if (i==0 && j==0)
                continue;
            /* preskacemo polja "van table" */
            if (v+i<0 || k+j<0 || v+i>=n || k+j>=n)
                continue;
            if (bombe[v+i][k+j]==1)
                br++;
         } 

   return br;

}

/* Centralna funkcija koja vrsi otvaranje polja i pritom se otvaranje "siri"
   i na polja koja su oko datog */

void otvori_polje(int v, int k)
{   /* Ukoliko smo "nagazili" bombu zavrsavamo program */
    if (bombe[v][k]==1)
    {   printf("BOOOOOOOOOOOOOOOOM!!!!\n");
        ispisi_bombe();
        exit(1);
    }
    else
    {    /* Brojimo bombe u okolini */
         int br=broj_bombi_u_okolini(v,k);
 
         /* Azuriramo stanje ovog polja */
         stanje[v][k]=(br==0)?PRAZNO:br;

         /* Ukoliko u okolini nema bombi, rekurzivno otvaramo
            sva polja u okolini koja su zatvorena */
         if (br==0)
         {   /* Petlje indeksiraju sva okolna polja */
             int i,j;
             for (i=-1; i<=1; i++)
                for (j=-1; j<=1; j++)
                {   /* Preskacemo centralno polje */
                    if (i==0 && j==0)
                        continue;
                    /* Preskacemo polja van table */
                    if (v+i<0 || v+i>=n || k+j<0 || k+j>=n)
                          continue;
                    /* Ukoliko je okolno polje zatvoreno, otvaramo ga */
                    if (stanje[v+i][k+j]==ZATVORENO)
                          otvori_polje(v+i, k+j);
                }
         }
    } 
}


/* Funkcija utrdjuje da li je partija gotova 
   Partija je gotova u trenutku kada su sve bombe pokrivene zastavicama i 
   kada nijedno drugo polje nije pokriveno zastavicom 
*/

int gotova_partija()
{  int i,j;
   for (i=0; i<n; i++)
      for (j=0; j<n; j++)
      {   /* Ukoliko postoji nepokrivena bomba, partija nije zavrsena */
          if (bombe[i][j]==1 && stanje[i][j]!=ZASTAVICA)
              return 0;
      }

  /* Partija je zavrsena samo ukoliko je broj zastavica jednak broj bombi */
  return broj_zastavica==broj_bombi;

}

main()
{ 

  /* Unosimo dimenziju table */
  printf("Unesite dimenziju table : ");
  scanf("%d",&n);


  /* Alociramo table */
  bombe=alociraj(n);
  stanje=alociraj(n);

  /* Postavljamo bombe */
  postavi_bombe();
 
  /* Sve dok partija nije gotova */
  while(!gotova_partija())
  {    int v,k;
       char akcija;

       /* Ispisujemo tekuce stanje */
       ispisi_stanje();

       /* Sve dok korisnik ne unese o ili z trazimo od njega da upise odgovarajucu akciju */
       do
       {  printf("Unesi akciju (o - otvaranje polja, z - postavljanje zastavice) : ");
	  while(isspace(akcija = getchar()));
       }  while (akcija!='o' && akcija!='z');
     
       /* Trazimo od korisnika da unese koordinate polja sve dok ih ne unese ispravno
          Korisnicke koordinate krecu od 1, a interne od 0 */
       do
       {
          printf("Unesi koordinate polja : ");
          scanf("%d",&v);
          scanf("%d",&k);
       } while(v<1 || v>n || k<1 || k>n);

       /* Reagujemo na akciju */
       switch(akcija)
       {   case 'o':
               otvori_polje(v-1,k-1);
               break;
           case 'z':
               postavi_zastavicu(v-1,k-1);
       }
  }

  /* Konstatujemo pobedu */
  ispisi_stanje();
  obrisi(stanje, n);
  obrisi(bombe, n);

  printf ("Cestitam! Pobedili ste\n");

}
