/* get_bits, set_bits, invert_bits - 
   izdvajanje, postavljanje i invertovanje pojedinacnih bitova */
#include <stdio.h>

/* Pomocna funkcija - stampa bitove neoznacenog broja */
void print_bits(unsigned x)
{
	int wl = sizeof(unsigned)*8;
	
	unsigned mask;
	for (mask = 1<<wl-1; mask; mask >>= 1)
		putchar(x&mask ? '1' : '0');
	
	putchar('\n');
}


/* Funkcija vraca n bitova broja x koji pocinju na poziciji p */
unsigned get_bits(unsigned x, int p, int n)
{
	/* Gradimo masku koja ima poslednjih n jedinica
		0000000...00011111
	   tako sto sve jedinice ~0 pomerimo u levo za n mesta
	        1111111...1100000
	   a zatim komplementiramo	
	*/
	unsigned last_n_1 = ~(~0 << n);
	
	/* x pomerimo u desno za odgovarajuci broj mesta, a zatim 
	   konjunkcijom sa konstruisanom maskom obrisemo pocetne cifre */
	   
	return (x >> p+1-n) & last_n_1;
}

/* Funkcija vraca modifikovano x tako sto mu je izmenjeno n bitova 
   pocevsi od pozicije p i na ta mesta je upisano poslednjih n bitova 
   broja y */
unsigned set_bits(unsigned x, int p, int n, unsigned y)
{
	/* Maska 000000...000111111  - poslednjih n jedinica */
	unsigned last_n_1 = ~(~0 << n);
	
	/* Maska 1111100..000111111 - n nula pocevsi od pozicije p */
	unsigned middle_n_0 = ~(last_n_1 << p+1-n);
	
	/* Brisemo n bitova pocevsi od pozicije p */
	x = x & middle_n_0;
	
	/* Izdvajamo poslednjih n bitova broja y i pomeramo ih na poziciju p */
	y = (y & last_n_1) << p+1-n;
	
	/* Upisujemo bitove broja y u broj x i vracamo rezultat */
	return x | y;
	
}

/* Invertuje n bitova broja x pocevsi od pozicije p */
unsigned invert_bits(unsigned x, int p, int n)
{
	/* Maska 000000111...1100000 - n jedinica pocevsi od pozicije p */
	unsigned middle_n_1 = ~(~0 << n) << p+1-n;
	
	/* Invertujemo koristeci ekskluzivnu disjunkciju */
	return x ^ middle_n_1;
}

main()
{
	unsigned x = 0x0AA0AFA0;
	print_bits(x);
	
	print_bits(get_bits(x, 15, 8));
	print_bits(set_bits(x, 15, 8, 0xFF));
	print_bits(invert_bits(x, 15, 8));
}

