/* Citanje niza struktura iz tektsualne datoteke - artikli prodavnice */

/* Datoteka cije se ime unosi sa standardnog ulaza sadrzi podatke o 
   proizvodima koji se prodaju u okviru odredjene prodavnice. 
   Svaki proizvod se odlikuje sledecim podacima :
	bar-kod   - petocifreni pozitivan broj
	ime       - niska karaktera
	cena      - realan broj zaokruzen na dve decimale 
	pdv       - stopa poreza - realan broj zaokruzen na dve decimale
   Pretpostavljamo da su podaci u datoteci korektno zadati.   

   Pretpostavljamo da se u prodavnici ne prodaje vise od 1000 razlicitih artikala.
   Na standardni izlaz ispisati podatke o svim proizvodima koji se prodaju.
*/
	
#include <stdio.h>

/* Maksimalna duzina imena proizvoda */
#define MAX_IME 30

/* Struktura za cuvanje podataka o jednom artiklu */
typedef struct _artikal
{
	int bar_kod; 
	char ime[MAX_IME];
	float cena;
	float pdv;
} artikal;

/* Maksimalni broj artikala */
#define MAX_ARTIKALA 1000

/* Niz struktura u kome se cuvaju podaci o artiklima */ 
artikal artikli[MAX_ARTIKALA];

/* Broj trenutno ucitanih artikala */
int br_artikala = 0;

/* Ucitava podatke o jednom artiklu iz date datoteke. 
   Vraca da li su podaci uspesno procitani */
int ucitaj_artikal(FILE* f, artikal* a)
{
	/* Citamo bar kod */
	fscanf(f, "%d", &(a->bar_kod));

	/* Ukoliko smo dosli do kraja datoteke prijavljujemo neuspeh */
	if (feof(f))
		return 0;

	/* Citamo ime proizvoda */
	fscanf(f, "%s", a->ime);
	/* Citamo cenu */
	fscanf(f, "%f", &(a->cena));
	/* Citamo stopu poreza */
	fscanf(f, "%f", &(a->pdv));
	
	/* Prijavljujemo uspeh */
	return 1;
}

/* Izracunava ukupnu cenu datog artikla */
float cena(artikal a)
{
	return  a.cena*(1+a.pdv);
}

/* Ispisuje podatke o svim artiklima */
void ispisi_artikle()
{
	int i;
	for (i = 0; i<br_artikala; i++)
		printf("%-5d  %-10s %.2f %.2f    = %.2f\n", 
			artikli[i].bar_kod, artikli[i].ime, 
			artikli[i].cena, artikli[i].pdv, cena(artikli[i]));
}

main(int argc, char* argv[])
{
	FILE* f;

	/* Ukoliko nije navedeno ime kao argument komandne linije, 
	   trazimo od korisnika da ga unese */
	if (argc<2)
	{
		/* Ucitavamo ime datoteke */
		char ime_datoteke[256];
		printf("U kojoj datoteci se nalaze podaci o proizvodima: ");
		scanf("%s", ime_datoteke);

		/* Otvaramo datoteku i proveravamo da li smo uspeli */
		if ( (f = fopen(ime_datoteke, "r")) == NULL)
		{
			printf("Greska : datoteka %s ne moze biti otvorena\n", 
				ime_datoteke);
		}
	}
	/* Ime datoteke je prvi argument komandne linije */
	else
	{

		/* Otvaramo datoteku i proveravamo da li smo uspeli */
		if ( (f = fopen(argv[1], "r")) == NULL)
		{
			printf("Greska : datoteka %s ne moze biti otvorena\n", 
				argv[1]);
		}

	}
		
	/* Ucitavamo artikle */
	while (ucitaj_artikal(f, &artikli[br_artikala]))
		br_artikala++;

	/* Ispisujemo podatke o svim artiklima */
	ispisi_artikle();

	/* Zatvaramo datoteku */
	fclose(f);
}


